/**
 * chroma.js - JavaScript library for color conversions
 *
 * Copyright (c) 2011-2019, Gregor Aisch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name Gregor Aisch may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL GREGOR AISCH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -------------------------------------------------------
 *
 * chroma.js includes colors from colorbrewer2.org, which are released under
 * the following license:
 *
 * Copyright (c) 2002 Cynthia Brewer, Mark Harrower,
 * and The Pennsylvania State University.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * ------------------------------------------------------
 *
 * Named colors are taken from X11 Color Names.
 * http://www.w3.org/TR/css3-color/#svg-color
 *
 * @preserve
 */

!function(r,e){"object"==typeof exports&&"undefined"!=typeof module?module.exports=e():"function"==typeof define&&define.amd?define(e):r.chroma=e()}(this,function(){"use strict";for(var n=function(r,e,n){return void 0===n&&(n=1),r<(e=void 0===e?0:e)?e:n<r?n:r},e={},r=0,t=["Boolean","Number","String","Function","Array","Date","RegExp","Undefined","Null"];r<t.length;r+=1){var a=t[r];e["[object "+a+"]"]=a.toLowerCase()}function $(r){return e[Object.prototype.toString.call(r)]||"object"}var f=Math.PI,o={clip_rgb:function(r){r._clipped=!1,r._unclipped=r.slice(0);for(var e=0;e<=3;e++)e<3?((r[e]<0||255<r[e])&&(r._clipped=!0),r[e]=n(r[e],0,255)):3===e&&(r[e]=n(r[e],0,1));return r},limit:n,type:$,unpack:function(e,r){return void 0===r&&(r=null),3<=e.length?Array.prototype.slice.call(e):"object"==$(e[0])&&r?r.split("").filter(function(r){return void 0!==e[0][r]}).map(function(r){return e[0][r]}):e[0]},last:function(r){if(r.length<2)return null;var e=r.length-1;return"string"==$(r[e])?r[e].toLowerCase():null},PI:f,TWOPI:2*f,PITHIRD:f/3,DEG2RAD:f/180,RAD2DEG:180/f},h={format:{},autodetect:[]},u=o.last,c=o.clip_rgb,i=o.type,l=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("object"===i(r[0])&&r[0].constructor&&r[0].constructor===this.constructor)return r[0];var n=!1;if(!(f=u(r))){n=!0,h.sorted||(h.autodetect=h.autodetect.sort(function(r,e){return e.p-r.p}),h.sorted=!0);for(var t=0,a=h.autodetect;t<a.length;t+=1){var f,o=a[t];if(f=o.test.apply(o,r))break}}if(!h.format[f])throw new Error("unknown format: "+r);n=h.format[f].apply(null,n?r:r.slice(0,-1));this._rgb=c(n),3===this._rgb.length&&this._rgb.push(1)};l.prototype.toString=function(){return"function"==i(this.hex)?this.hex():"["+this._rgb.join(",")+"]"};var p=l,d=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(d.Color,[null].concat(r)))};d.Color=p,d.version="2.1.1";var N=d,s=o.unpack,b=Math.max,g=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=s(r,"rgb"),t=n[0],a=n[1],f=n[2],o=1-b(t/=255,b(a/=255,f/=255)),n=o<1?1/(1-o):0;return[(1-t-o)*n,(1-a-o)*n,(1-f-o)*n,o]},v=o.unpack,m=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=(r=v(r,"cmyk"))[0],t=r[1],a=r[2],f=r[3],o=4<r.length?r[4]:1;return 1===f?[0,0,0,o]:[1<=n?0:255*(1-n)*(1-f),1<=t?0:255*(1-t)*(1-f),1<=a?0:255*(1-a)*(1-f),o]},y=o.unpack,w=o.type;p.prototype.cmyk=function(){return g(this._rgb)},N.cmyk=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["cmyk"])))},h.format.cmyk=m,h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=y(r,"cmyk"),"array"===w(r)&&4===r.length)return"cmyk"}});function k(r){return Math.round(100*r)/100}var M=o.unpack,_=o.last,x=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=M(r,"hsla"),t=_(r)||"lsa";return n[0]=k(n[0]||0),n[1]=k(100*n[1])+"%",n[2]=k(100*n[2])+"%","hsla"===t||3<n.length&&n[3]<1?(n[3]=3<n.length?n[3]:1,t="hsla"):n.length=3,t+"("+n.join(",")+")"},A=o.unpack,E=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=(r=A(r,"rgba"))[0],t=r[1],a=r[2];n/=255,t/=255,a/=255;var f,o,u=Math.min(n,t,a),c=Math.max(n,t,a),i=(c+u)/2;return c===u?(f=0,o=Number.NaN):f=i<.5?(c-u)/(c+u):(c-u)/(2-c-u),n==c?o=(t-a)/(c-u):t==c?o=2+(a-n)/(c-u):a==c&&(o=4+(n-t)/(c-u)),(o*=60)<0&&(o+=360),3<r.length&&void 0!==r[3]?[o,f,i,r[3]]:[o,f,i]},P=o.unpack,F=o.last,O=Math.round,j=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=P(r,"rgba"),t=F(r)||"rgb";return"hsl"==t.substr(0,3)?x(E(n),t):(n[0]=O(n[0]),n[1]=O(n[1]),n[2]=O(n[2]),("rgba"===t||3<n.length&&n[3]<1)&&(n[3]=3<n.length?n[3]:1,t="rgba"),t+"("+n.slice(0,"rgb"===t?3:4).join(",")+")")},G=o.unpack,q=Math.round,L=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n,t,a,f=(r=G(r,"hsl"))[0],o=r[1],u=r[2];if(0===o)n=t=a=255*u;else{var c=[0,0,0],i=[0,0,0],l=u<.5?u*(1+o):u+o-u*o,h=2*u-l,f=f/360;c[0]=f+1/3,c[1]=f,c[2]=f-1/3;for(var d=0;d<3;d++)c[d]<0&&(c[d]+=1),1<c[d]&&--c[d],6*c[d]<1?i[d]=h+6*(l-h)*c[d]:2*c[d]<1?i[d]=l:3*c[d]<2?i[d]=h+(l-h)*(2/3-c[d])*6:i[d]=h;n=(f=[q(255*i[0]),q(255*i[1]),q(255*i[2])])[0],t=f[1],a=f[2]}return 3<r.length?[n,t,a,r[3]]:[n,t,a,1]},R=/^rgb\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/,I=/^rgba\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*,\s*([01]|[01]?\.\d+)\)$/,B=/^rgb\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,C=/^rgba\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,D=/^hsl\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,S=/^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,Y=Math.round,f=function(r){var e;if(r=r.toLowerCase().trim(),h.format.named)try{return h.format.named(r)}catch(r){}if(e=r.match(R)){for(var n=e.slice(1,4),t=0;t<3;t++)n[t]=+n[t];return n[3]=1,n}if(e=r.match(I)){for(var a=e.slice(1,5),f=0;f<4;f++)a[f]=+a[f];return a}if(e=r.match(B)){for(var o=e.slice(1,4),u=0;u<3;u++)o[u]=Y(2.55*o[u]);return o[3]=1,o}if(e=r.match(C)){for(var c=e.slice(1,5),i=0;i<3;i++)c[i]=Y(2.55*c[i]);return c[3]=+c[3],c}if(e=r.match(D)){var l=e.slice(1,4);l[1]*=.01,l[2]*=.01;l=L(l);return l[3]=1,l}if(e=r.match(S)){r=e.slice(1,4);r[1]*=.01,r[2]*=.01;r=L(r);return r[3]=+e[4],r}};f.test=function(r){return R.test(r)||I.test(r)||B.test(r)||C.test(r)||D.test(r)||S.test(r)};var z=f,T=o.type;p.prototype.css=function(r){return j(this._rgb,r)},N.css=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["css"])))},h.format.css=z,h.autodetect.push({p:5,test:function(r){for(var e=[],n=arguments.length-1;0<n--;)e[n]=arguments[n+1];if(!e.length&&"string"===T(r)&&z.test(r))return"css"}});var U=o.unpack;h.format.gl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=U(r,"rgba");return n[0]*=255,n[1]*=255,n[2]*=255,n},N.gl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["gl"])))},p.prototype.gl=function(){var r=this._rgb;return[r[0]/255,r[1]/255,r[2]/255,r[3]]};var V=o.unpack,W=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n,t=V(r,"rgb"),a=t[0],f=t[1],o=t[2],u=Math.min(a,f,o),c=Math.max(a,f,o),t=c-u;return 0==t?n=Number.NaN:(a===c&&(n=(f-o)/t),f===c&&(n=2+(o-a)/t),o===c&&(n=4+(a-f)/t),(n*=60)<0&&(n+=360)),[n,100*t/255,u/(255-t)*100]},X=o.unpack,H=Math.floor,l=function(){for(var r,e,n,t=[],a=arguments.length;a--;)t[a]=arguments[a];var f,o,u,c=(t=X(t,"hcg"))[0],i=t[1],l=t[2];l*=255;var h=255*i;if(0===i)f=o=u=l;else{360<(c=360===c?0:c)&&(c-=360),c<0&&(c+=360);var d=H(c/=60),c=c-d,s=l*(1-i),b=s+h*(1-c),p=s+h*c,g=s+h;switch(d){case 0:f=(r=[g,p,s])[0],o=r[1],u=r[2];break;case 1:f=(r=[b,g,s])[0],o=r[1],u=r[2];break;case 2:f=(e=[s,g,p])[0],o=e[1],u=e[2];break;case 3:f=(e=[s,b,g])[0],o=e[1],u=e[2];break;case 4:f=(n=[p,s,g])[0],o=n[1],u=n[2];break;case 5:f=(n=[g,s,b])[0],o=n[1],u=n[2]}}return[f,o,u,3<t.length?t[3]:1]},J=o.unpack,K=o.type;p.prototype.hcg=function(){return W(this._rgb)},N.hcg=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hcg"])))},h.format.hcg=l,h.autodetect.push({p:1,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=J(r,"hcg"),"array"===K(r)&&3===r.length)return"hcg"}});var Q=o.unpack,Z=o.last,rr=Math.round,er=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=Q(r,"rgba"),t=n[0],a=n[1],f=n[2],o=n[3],n=Z(r)||"auto";void 0===o&&(o=1),"auto"===n&&(n=o<1?"rgba":"rgb");var u=(u="000000"+((t=rr(t))<<16|(a=rr(a))<<8|(f=rr(f))).toString(16)).substr(u.length-6),c=(c="0"+rr(255*o).toString(16)).substr(c.length-2);switch(n.toLowerCase()){case"rgba":return"#"+u+c;case"argb":return"#"+c+u;default:return"#"+u}},nr=/^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/,tr=/^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/,ar=function(r){if(r.match(nr)){3===(r=4===r.length||7===r.length?r.substr(1):r).length&&(r=(r=r.split(""))[0]+r[0]+r[1]+r[1]+r[2]+r[2]);var e=parseInt(r,16);return[e>>16,e>>8&255,255&e,1]}if(r.match(tr)){4===(r=5===r.length||9===r.length?r.substr(1):r).length&&(r=(r=r.split(""))[0]+r[0]+r[1]+r[1]+r[2]+r[2]+r[3]+r[3]);e=parseInt(r,16);return[e>>24&255,e>>16&255,e>>8&255,Math.round((255&e)/255*100)/100]}throw new Error("unknown hex color: "+r)},fr=o.type;p.prototype.hex=function(r){return er(this._rgb,r)},N.hex=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hex"])))},h.format.hex=ar,h.autodetect.push({p:4,test:function(r){for(var e=[],n=arguments.length-1;0<n--;)e[n]=arguments[n+1];if(!e.length&&"string"===fr(r)&&0<=[3,4,5,6,7,8,9].indexOf(r.length))return"hex"}});var or=o.unpack,ur=o.TWOPI,cr=Math.min,ir=Math.sqrt,lr=Math.acos,hr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n,t=or(r,"rgb"),a=t[0],f=t[1],o=t[2],u=cr(a/=255,f/=255,o/=255),t=(a+f+o)/3,u=0<t?1-u/t:0;return 0==u?n=NaN:(n=(a-f+(a-o))/2,n/=ir((a-f)*(a-f)+(a-o)*(f-o)),n=lr(n),f<o&&(n=ur-n),n/=ur),[360*n,u,t]},dr=o.unpack,sr=o.limit,br=o.TWOPI,pr=o.PITHIRD,gr=Math.cos,m=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n,t,a,f=(r=dr(r,"hsi"))[0],o=r[1],u=r[2];return isNaN(f)&&(f=0),isNaN(o)&&(o=0),360<f&&(f-=360),f<0&&(f+=360),(f/=360)<1/3?t=1-((a=(1-o)/3)+(n=(1+o*gr(br*f)/gr(pr-br*f))/3)):f<2/3?a=1-((n=(1-o)/3)+(t=(1+o*gr(br*(f-=1/3))/gr(pr-br*f))/3)):n=1-((t=(1-o)/3)+(a=(1+o*gr(br*(f-=2/3))/gr(pr-br*f))/3)),[255*(n=sr(u*n*3)),255*(t=sr(u*t*3)),255*(a=sr(u*a*3)),3<r.length?r[3]:1]},vr=o.unpack,mr=o.type;p.prototype.hsi=function(){return hr(this._rgb)},N.hsi=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hsi"])))},h.format.hsi=m,h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=vr(r,"hsi"),"array"===mr(r)&&3===r.length)return"hsi"}});var yr=o.unpack,wr=o.type;p.prototype.hsl=function(){return E(this._rgb)},N.hsl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hsl"])))},h.format.hsl=L,h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=yr(r,"hsl"),"array"===wr(r)&&3===r.length)return"hsl"}});var kr=o.unpack,Mr=Math.min,Nr=Math.max,_r=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n,t,a=(r=kr(r,"rgb"))[0],f=r[1],o=r[2],u=Mr(a,f,o),c=Nr(a,f,o),u=c-u;return 0===c?(n=Number.NaN,t=0):(t=u/c,a===c&&(n=(f-o)/u),f===c&&(n=2+(o-a)/u),o===c&&(n=4+(a-f)/u),(n*=60)<0&&(n+=360)),[n,t,c/255]},xr=o.unpack,Ar=Math.floor,f=function(){for(var r,e,n,t=[],a=arguments.length;a--;)t[a]=arguments[a];var f,o,u,c=(t=xr(t,"hsv"))[0],i=t[1],l=t[2];if(l*=255,0===i)f=o=u=l;else{360<(c=360===c?0:c)&&(c-=360),c<0&&(c+=360);var h=Ar(c/=60),c=c-h,d=l*(1-i),s=l*(1-i*c),b=l*(1-i*(1-c));switch(h){case 0:f=(r=[l,b,d])[0],o=r[1],u=r[2];break;case 1:f=(r=[s,l,d])[0],o=r[1],u=r[2];break;case 2:f=(e=[d,l,b])[0],o=e[1],u=e[2];break;case 3:f=(e=[d,s,l])[0],o=e[1],u=e[2];break;case 4:f=(n=[b,d,l])[0],o=n[1],u=n[2];break;case 5:f=(n=[l,d,s])[0],o=n[1],u=n[2]}}return[f,o,u,3<t.length?t[3]:1]},Er=o.unpack,Pr=o.type;p.prototype.hsv=function(){return _r(this._rgb)},N.hsv=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hsv"])))},h.format.hsv=f,h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=Er(r,"hsv"),"array"===Pr(r)&&3===r.length)return"hsv"}});function Fr(r){return(r/=255)<=.04045?r/12.92:Sr((r+.055)/1.055,2.4)}function Or(r){return Cr<r?Sr(r,1/3):r/Br+Rr}var jr=18,Gr=.95047,qr=1,Lr=1.08883,Rr=.137931034,Ir=.206896552,Br=.12841855,Cr=.008856452,Dr=o.unpack,Sr=Math.pow,$r=function(r,e,n){return r=Fr(r),e=Fr(e),n=Fr(n),[Or((.4124564*r+.3575761*e+.1804375*n)/Gr),Or((.2126729*r+.7151522*e+.072175*n)/qr),Or((.0193339*r+.119192*e+.9503041*n)/Lr)]},Yr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=Dr(r,"rgb"),t=n[0],a=n[1],f=n[2],n=$r(t,a,f),t=n[0],a=n[1],f=116*a-16;return[f<0?0:f,500*(t-a),200*(a-n[2])]},zr=o.unpack,Tr=Math.pow,Ur=function(r){return 255*(r<=.00304?12.92*r:1.055*Tr(r,1/2.4)-.055)},Vr=function(r){return Ir<r?r*r*r:Br*(r-Rr)},Wr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=(r=zr(r,"lab"))[0],t=r[1],a=r[2],n=(n+16)/116,t=isNaN(t)?n:n+t/500,a=isNaN(a)?n:n-a/200;return n=qr*Vr(n),t=Gr*Vr(t),a=Lr*Vr(a),[Ur(3.2404542*t-1.5371385*n-.4985314*a),Ur(-.969266*t+1.8760108*n+.041556*a),Ur(.0556434*t-.2040259*n+1.0572252*a),3<r.length?r[3]:1]},Xr=o.unpack,Hr=o.type;p.prototype.lab=function(){return Yr(this._rgb)},N.lab=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["lab"])))},h.format.lab=Wr,h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=Xr(r,"lab"),"array"===Hr(r)&&3===r.length)return"lab"}});var Jr=o.unpack,Kr=o.RAD2DEG,Qr=Math.sqrt,Zr=Math.atan2,re=Math.round,ee=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=Jr(r,"lab"),t=n[0],a=n[1],f=n[2],n=Qr(a*a+f*f),a=(Zr(f,a)*Kr+360)%360;return[t,n,a=0===re(1e4*n)?Number.NaN:a]},ne=o.unpack,te=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=ne(r,"rgb"),t=n[0],a=n[1],n=n[2],t=Yr(t,a,n),a=t[0],n=t[1],t=t[2];return ee(a,n,t)},ae=o.unpack,fe=o.DEG2RAD,oe=Math.sin,ue=Math.cos,ce=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=ae(r,"lch"),t=n[0],a=n[1],n=n[2];return isNaN(n)&&(n=0),[t,ue(n*=fe)*a,oe(n)*a]},ie=o.unpack,le=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=(r=ie(r,"lch"))[0],t=r[1],a=r[2],n=ce(n,t,a),t=n[0],a=n[1],n=n[2],n=Wr(t,a,n);return[n[0],n[1],n[2],3<r.length?r[3]:1]},he=o.unpack,l=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=he(r,"hcl").reverse();return le.apply(void 0,n)},de=o.unpack,se=o.type;p.prototype.lch=function(){return te(this._rgb)},p.prototype.hcl=function(){return te(this._rgb).reverse()},N.lch=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["lch"])))},N.hcl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["hcl"])))},h.format.lch=le,h.format.hcl=l,["lch","hcl"].forEach(function(n){return h.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=de(r,n),"array"===se(r)&&3===r.length)return n}})});var be={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflower:"#6495ed",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",darkgrey:"#a9a9a9",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",green:"#008000",greenyellow:"#adff2f",grey:"#808080",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",laserlemon:"#ffff54",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrod:"#fafad2",lightgoldenrodyellow:"#fafad2",lightgray:"#d3d3d3",lightgreen:"#90ee90",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",maroon2:"#7f0000",maroon3:"#b03060",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",purple2:"#7f007f",purple3:"#a020f0",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"},pe=o.type;p.prototype.name=function(){for(var r=er(this._rgb,"rgb"),e=0,n=Object.keys(be);e<n.length;e+=1){var t=n[e];if(be[t]===r)return t.toLowerCase()}return r},h.format.named=function(r){if(r=r.toLowerCase(),be[r])return ar(be[r]);throw new Error("unknown color name: "+r)},h.autodetect.push({p:5,test:function(r){for(var e=[],n=arguments.length-1;0<n--;)e[n]=arguments[n+1];if(!e.length&&"string"===pe(r)&&be[r.toLowerCase()])return"named"}});var ge=o.unpack,ve=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=ge(r,"rgb");return(n[0]<<16)+(n[1]<<8)+n[2]},me=o.type,m=function(r){if("number"==me(r)&&0<=r&&r<=16777215)return[r>>16,r>>8&255,255&r,1];throw new Error("unknown num color: "+r)},ye=o.type;p.prototype.num=function(){return ve(this._rgb)},N.num=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["num"])))},h.format.num=m,h.autodetect.push({p:5,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(1===r.length&&"number"===ye(r[0])&&0<=r[0]&&r[0]<=16777215)return"num"}});var we=o.unpack,ke=o.type,Me=Math.round;p.prototype.rgb=function(r){return!1===(r=void 0===r?!0:r)?this._rgb.slice(0,3):this._rgb.slice(0,3).map(Me)},p.prototype.rgba=function(n){return void 0===n&&(n=!0),this._rgb.slice(0,4).map(function(r,e){return!(e<3)||!1===n?r:Me(r)})},N.rgb=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["rgb"])))},h.format.rgb=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=we(r,"rgba");return void 0===n[3]&&(n[3]=1),n},h.autodetect.push({p:3,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(r=we(r,"rgba"),"array"===ke(r)&&(3===r.length||4===r.length&&"number"==ke(r[3])&&0<=r[3]&&r[3]<=1))return"rgb"}});var Ne=Math.log,_e=function(r){var e,n,r=r/100,t=r<66?(e=255,n=-155.25485562709179-.44596950469579133*(n=r-2)+104.49216199393888*Ne(n),r<20?0:.8274096064007395*(t=r-10)-254.76935184120902+115.67994401066147*Ne(t)):(e=351.97690566805693+.114206453784165*(e=r-55)-40.25366309332127*Ne(e),n=325.4494125711974+.07943456536662342*(n=r-50)-28.0852963507957*Ne(n),255);return[e,n,t,1]},xe=o.unpack,Ae=Math.round,Ee=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];for(var n=xe(r,"rgb"),t=n[0],a=n[2],f=1e3,o=4e4;.4<o-f;){var u,c=_e(u=.5*(o+f));c[2]/c[0]>=a/t?o=u:f=u}return Ae(u)};p.prototype.temp=p.prototype.kelvin=p.prototype.temperature=function(){return Ee(this._rgb)},N.temp=N.kelvin=N.temperature=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(p,[null].concat(r,["temp"])))},h.format.temp=h.format.kelvin=h.format.temperature=_e;var Pe=o.type;p.prototype.alpha=function(r,e){return void 0===e&&(e=!1),void 0!==r&&"number"===Pe(r)?e?(this._rgb[3]=r,this):new p([this._rgb[0],this._rgb[1],this._rgb[2],r],"rgb"):this._rgb[3]},p.prototype.clipped=function(){return this._rgb._clipped||!1},p.prototype.darken=function(r){void 0===r&&(r=1);var e=this.lab();return e[0]-=jr*r,new p(e,"lab").alpha(this.alpha(),!0)},p.prototype.brighten=function(r){return this.darken(-(r=void 0===r?1:r))},p.prototype.darker=p.prototype.darken,p.prototype.brighter=p.prototype.brighten,p.prototype.get=function(r){var e=r.split("."),n=e[0],t=e[1],r=this[n]();if(t){e=n.indexOf(t);if(-1<e)return r[e];throw new Error("unknown channel "+t+" in mode "+n)}return r};var Fe=o.type,Oe=Math.pow;p.prototype.luminance=function(a){if(void 0===a||"number"!==Fe(a))return Ge.apply(void 0,this._rgb.slice(0,3));if(0===a)return new p([0,0,0,this._rgb[3]],"rgb");if(1===a)return new p([255,255,255,this._rgb[3]],"rgb");var r=this.luminance(),f=20,o=function(r,e){var n=r.interpolate(e,.5,"rgb"),t=n.luminance();return Math.abs(a-t)<1e-7||!f--?n:a<t?o(r,n):o(n,e)},r=(a<r?o(new p([0,0,0]),this):o(this,new p([255,255,255]))).rgb();return new p(r.concat([this._rgb[3]]))};function je(r,e,n){void 0===n&&(n=.5);for(var t=[],a=arguments.length-3;0<a--;)t[a]=arguments[a+3];var f=t[0]||"lrgb";if(Le[f]||t.length||(f=Object.keys(Le)[0]),!Le[f])throw new Error("interpolation mode "+f+" is not defined");return"object"!==Re(r)&&(r=new p(r)),"object"!==Re(e)&&(e=new p(e)),Le[f](r,e,n).alpha(r.alpha()+n*(e.alpha()-r.alpha()))}var Ge=function(r,e,n){return.2126*(r=qe(r))+.7152*(e=qe(e))+.0722*(n=qe(n))},qe=function(r){return(r/=255)<=.03928?r/12.92:Oe((r+.055)/1.055,2.4)},Le={},Re=o.type;p.prototype.mix=p.prototype.interpolate=function(r,e){void 0===e&&(e=.5);for(var n=[],t=arguments.length-2;0<t--;)n[t]=arguments[t+2];return je.apply(void 0,[this,r,e].concat(n))},p.prototype.premultiply=function(r){var e=this._rgb,n=e[3];return(r=void 0===r?!1:r)?(this._rgb=[e[0]*n,e[1]*n,e[2]*n,n],this):new p([e[0]*n,e[1]*n,e[2]*n,n],"rgb")},p.prototype.saturate=function(r){void 0===r&&(r=1);var e=this.lch();return e[1]+=jr*r,e[1]<0&&(e[1]=0),new p(e,"lch").alpha(this.alpha(),!0)},p.prototype.desaturate=function(r){return this.saturate(-(r=void 0===r?1:r))};var Ie=o.type;p.prototype.set=function(r,e,n){void 0===n&&(n=!1);var t=r.split("."),a=t[0],r=t[1],f=this[a]();if(r){var o=a.indexOf(r);if(-1<o){if("string"==Ie(e))switch(e.charAt(0)){case"+":case"-":f[o]+=+e;break;case"*":f[o]*=+e.substr(1);break;case"/":f[o]/=+e.substr(1);break;default:f[o]=+e}else{if("number"!==Ie(e))throw new Error("unsupported value for Color.set");f[o]=e}t=new p(f,a);return n?(this._rgb=t._rgb,this):t}throw new Error("unknown channel "+r+" in mode "+a)}return f};Le.rgb=function(r,e,n){r=r._rgb,e=e._rgb;return new p(r[0]+n*(e[0]-r[0]),r[1]+n*(e[1]-r[1]),r[2]+n*(e[2]-r[2]),"rgb")};var Be=Math.sqrt,Ce=Math.pow;Le.lrgb=function(r,e,n){var t=r._rgb,a=t[0],f=t[1],o=t[2],r=e._rgb,t=r[0],e=r[1],r=r[2];return new p(Be(Ce(a,2)*(1-n)+Ce(t,2)*n),Be(Ce(f,2)*(1-n)+Ce(e,2)*n),Be(Ce(o,2)*(1-n)+Ce(r,2)*n),"rgb")};Le.lab=function(r,e,n){r=r.lab(),e=e.lab();return new p(r[0]+n*(e[0]-r[0]),r[1]+n*(e[1]-r[1]),r[2]+n*(e[2]-r[2]),"lab")};function De(r,e,n,t){var a,f,o,u,c,i,l,h,d,s;return"hsl"===t?(a=r.hsl(),f=e.hsl()):"hsv"===t?(a=r.hsv(),f=e.hsv()):"hcg"===t?(a=r.hcg(),f=e.hcg()):"hsi"===t?(a=r.hsi(),f=e.hsi()):"lch"!==t&&"hcl"!==t||(t="hcl",a=r.hcl(),f=e.hcl()),"h"===t.substr(0,1)&&(o=a[0],c=a[1],l=a[2],u=f[0],i=f[1],h=f[2]),isNaN(o)||isNaN(u)?isNaN(o)?isNaN(u)?s=Number.NaN:(s=u,1!=l&&0!=l||"hsv"==t||(d=i)):(s=o,1!=h&&0!=h||"hsv"==t||(d=c)):s=o+n*(o<u&&180<u-o?u-(o+360):u<o&&180<o-u?u+360-o:u-o),new p([s,d=void 0===d?c+n*(i-c):d,l+n*(h-l)],t)}f=function(r,e,n){return De(r,e,n,"lch")};Le.lch=f,Le.hcl=f;Le.num=function(r,e,n){r=r.num(),e=e.num();return new p(r+n*(e-r),"num")};Le.hcg=function(r,e,n){return De(r,e,n,"hcg")};Le.hsi=function(r,e,n){return De(r,e,n,"hsi")};Le.hsl=function(r,e,n){return De(r,e,n,"hsl")};Le.hsv=function(r,e,n){return De(r,e,n,"hsv")};function Se(i){function n(r){if((r=r||["#fff","#000"])&&"string"===He(r)&&N.brewer&&N.brewer[r.toLowerCase()]&&(r=N.brewer[r.toLowerCase()]),"array"===He(r)){r=(r=1===r.length?[r[0],r[0]]:r).slice(0);for(var e=0;e<r.length;e++)r[e]=N(r[e]);for(var n=c.length=0;n<r.length;n++)c.push(n/(r.length-1))}return f(),b=r}function l(r,e){var n,t;if(null==e&&(e=!1),isNaN(r)||null===r)return u;if(t=e?r:s&&2<s.length?function(r){if(null==s)return 0;for(var e=s.length-1,n=0;n<e&&r>=s[n];)n++;return n-1}(r)/(s.length-2):g!==p?(r-p)/(g-p):1,t=k(t),e||(t=w(t)),1!==y&&(t=Je(t,y)),t=d[0]+t*(1-d[0]-d[1]),t=Math.min(1,Math.max(0,t)),e=Math.floor(1e4*t),m&&v[e])n=v[e];else{if("array"===He(b))for(var a=0;a<c.length;a++){var f=c[a];if(t<=f){n=b[a];break}if(f<=t&&a===c.length-1){n=b[a];break}if(f<t&&t<c[a+1]){t=(t-f)/(c[a+1]-f),n=N.interpolate(b[a],b[a+1],t,o);break}}else"function"===He(b)&&(n=b(t));m&&(v[e]=n)}return n}var e,o="rgb",u=N("#ccc"),t=0,h=[0,1],c=[],d=[0,0],s=!1,b=[],a=!1,p=0,g=1,v={},m=!0,y=1,w=function(r){return r},k=function(r){return r},f=function(){return v={}};function M(r){return r=N(l(r)),a&&r[a]?r[a]():r}return n(i),M.classes=function(r){return null==r?s:("array"===He(r)?h=[(s=r)[0],r[r.length-1]]:(e=N.analyze(h),s=0===r?[e.min,e.max]:N.limits(e,"e",r)),M);var e},M.domain=function(n){if(!arguments.length)return h;p=n[0],g=n[n.length-1],c=[];var r=b.length;if(n.length===r&&p!==g)for(var e=0,t=Array.from(n);e<t.length;e+=1){var a=t[e];c.push((a-p)/(g-p))}else{for(var f,o,u=0;u<r;u++)c.push(u/(r-1));2<n.length&&(f=n.map(function(r,e){return e/(n.length-1)}),(o=n.map(function(r){return(r-p)/(g-p)})).every(function(r,e){return f[e]===r})||(k=function(r){if(r<=0||1<=r)return r;for(var e=0;r>=o[e+1];)e++;var n=(r-o[e])/(o[e+1]-o[e]);return f[e]+n*(f[e+1]-f[e])}))}return h=[p,g],M},M.mode=function(r){return arguments.length?(o=r,f(),M):o},M.range=function(r,e){return n(r),M},M.out=function(r){return a=r,M},M.spread=function(r){return arguments.length?(t=r,M):t},M.correctLightness=function(r){return e=r=null==r?!0:r,f(),w=e?function(r){for(var e=l(0,!0).lab()[0],n=l(1,!0).lab()[0],t=n<e,a=l(r,!0).lab()[0],f=e+(n-e)*r,o=a-f,u=0,c=1,i=20;.01<Math.abs(o)&&0<i--;)t&&(o*=-1),r+=o<0?.5*(c-(u=r)):.5*(u-(c=r)),a=l(r,!0).lab()[0],o=a-f;return r}:function(r){return r},M},M.padding=function(r){return null!=r?("number"===He(r)&&(r=[r,r]),d=r,M):d},M.colors=function(e,n){arguments.length<2&&(n="hex");var r=[];if(0===arguments.length)r=b.slice(0);else if(1===e)r=[M(.5)];else if(1<e)var t=h[0],a=h[1]-t,r=function(r,e,n){for(var t=[],a=r<e,f=n?a?e+1:e-1:e,o=r;a?o<f:f<o;a?o++:o--)t.push(o);return t}(0,e,!1).map(function(r){return M(t+r/(e-1)*a)});else{i=[];var f=[];if(s&&2<s.length)for(var o=1,u=s.length,c=1<=u;c?o<u:u<o;c?o++:o--)f.push(.5*(s[o-1]+s[o]));else f=h;r=f.map(M)}return r=N[n]?r.map(function(r){return r[n]()}):r},M.cache=function(r){return null!=r?(m=r,M):m},M.gamma=function(r){return null!=r?(y=r,M):y},M.nodata=function(r){return null!=r?(u=N(r),M):u},M}var $e=o.clip_rgb,Ye=Math.pow,ze=Math.sqrt,Te=Math.PI,Ue=Math.cos,Ve=Math.sin,We=Math.atan2,Xe=function(r,e){for(var n=r.length,t=[0,0,0,0],a=0;a<r.length;a++){var f=r[a],o=e[a]/n,f=f._rgb;t[0]+=Ye(f[0],2)*o,t[1]+=Ye(f[1],2)*o,t[2]+=Ye(f[2],2)*o,t[3]+=f[3]*o}return t[0]=ze(t[0]),t[1]=ze(t[1]),t[2]=ze(t[2]),.9999999<t[3]&&(t[3]=1),new p($e(t))},He=o.type,Je=Math.pow;var Ke=function(r){var e,n,t,a,f,o,u,c;return 2===(r=r.map(function(r){return new p(r)})).length?(o=r.map(function(r){return r.lab()}),n=o[0],t=o[1],o=function(e){var r=[0,1,2].map(function(r){return n[r]+e*(t[r]-n[r])});return new p(r,"lab")}):3===r.length?(e=r.map(function(r){return r.lab()}),n=e[0],t=e[1],a=e[2],o=function(e){var r=[0,1,2].map(function(r){return(1-e)*(1-e)*n[r]+2*(1-e)*e*t[r]+e*e*a[r]});return new p(r,"lab")}):4===r.length?(n=(e=r.map(function(r){return r.lab()}))[0],t=e[1],a=e[2],f=e[3],o=function(e){var r=[0,1,2].map(function(r){return(1-e)*(1-e)*(1-e)*n[r]+3*(1-e)*(1-e)*e*t[r]+3*(1-e)*e*e*a[r]+e*e*e*f[r]});return new p(r,"lab")}):5===r.length&&(u=Ke(r.slice(0,3)),c=Ke(r.slice(2,5)),o=function(r){return r<.5?u(2*r):c(2*(r-.5))}),o},Qe=function(r,e,n){if(!Qe[n])throw new Error("unknown blend mode "+n);return Qe[n](r,e)},l=function(n){return function(r,e){e=N(e).rgb(),r=N(r).rgb();return N.rgb(n(e,r))}},m=function(t){return function(r,e){var n=[];return n[0]=t(r[0],e[0]),n[1]=t(r[1],e[1]),n[2]=t(r[2],e[2]),n}};Qe.normal=l(m(function(r){return r})),Qe.multiply=l(m(function(r,e){return r*e/255})),Qe.screen=l(m(function(r,e){return 255*(1-(1-r/255)*(1-e/255))})),Qe.overlay=l(m(function(r,e){return e<128?2*r*e/255:255*(1-2*(1-r/255)*(1-e/255))})),Qe.darken=l(m(function(r,e){return e<r?e:r})),Qe.lighten=l(m(function(r,e){return e<r?r:e})),Qe.dodge=l(m(function(r,e){return 255===r||255<(r=e/255*255/(1-r/255))?255:r})),Qe.burn=l(m(function(r,e){return 255*(1-(1-e/255)/(r/255))}));for(var f=Qe,Ze=o.type,rn=o.clip_rgb,en=o.TWOPI,nn=Math.pow,tn=Math.sin,an=Math.cos,fn=Math.floor,on=Math.random,un=Math.log,cn=Math.pow,ln=Math.floor,hn=Math.abs,dn=function(r,e){void 0===e&&(e=null);var n={min:Number.MAX_VALUE,max:-1*Number.MAX_VALUE,sum:0,values:[],count:0};return(r="object"===$(r)?Object.values(r):r).forEach(function(r){null==(r=e&&"object"===$(r)?r[e]:r)||isNaN(r)||(n.values.push(r),n.sum+=r,r<n.min&&(n.min=r),r>n.max&&(n.max=r),n.count+=1)}),n.domain=[n.min,n.max],n.limits=function(r,e){return sn(n,r,e)},n},sn=function(r,e,n){void 0===e&&(e="equal"),void 0===n&&(n=7);var t=(r="array"==$(r)?dn(r):r).min,a=r.max,f=r.values.sort(function(r,e){return r-e});if(1===n)return[t,a];var o=[];if("c"===e.substr(0,1)&&(o.push(t),o.push(a)),"e"===e.substr(0,1)){o.push(t);for(var u=1;u<n;u++)o.push(t+u/n*(a-t));o.push(a)}else if("l"===e.substr(0,1)){if(t<=0)throw new Error("Logarithmic scales are only possible for values > 0");var c=Math.LOG10E*un(t),i=Math.LOG10E*un(a);o.push(t);for(var l=1;l<n;l++)o.push(cn(10,c+l/n*(i-c)));o.push(a)}else if("q"===e.substr(0,1)){o.push(t);for(var h=1;h<n;h++){var d=(f.length-1)*h/n,s=ln(d);s===d?o.push(f[s]):o.push(f[s]*(1-(d=d-s))+f[s+1]*d)}o.push(a)}else if("k"===e.substr(0,1)){var b,p=f.length,g=new Array(p),v=new Array(n),m=!0,y=0,w=null;(w=[]).push(t);for(var k=1;k<n;k++)w.push(t+k/n*(a-t));for(w.push(a);m;){for(var M=0;M<n;M++)v[M]=0;for(var N=0;N<p;N++)for(var _=f[N],x=Number.MAX_VALUE,A=void 0,E=0;E<n;E++){var P=hn(w[E]-_);P<x&&(x=P,A=E),v[A]++,g[N]=A}for(var F=new Array(n),O=0;O<n;O++)F[O]=null;for(var j=0;j<p;j++)null===F[b=g[j]]?F[b]=f[j]:F[b]+=f[j];for(var G=0;G<n;G++)F[G]*=1/v[G];for(var m=!1,q=0;q<n;q++)if(F[q]!==w[q]){m=!0;break}w=F,200<++y&&(m=!1)}for(var L={},R=0;R<n;R++)L[R]=[];for(var I=0;I<p;I++)L[b=g[I]].push(f[I]);for(var B=[],C=0;C<n;C++)B.push(L[C][0]),B.push(L[C][L[C].length-1]);B=B.sort(function(r,e){return r-e}),o.push(B[0]);for(var D=1;D<B.length;D+=2){var S=B[D];isNaN(S)||-1!==o.indexOf(S)||o.push(S)}}return o},l={analyze:dn,limits:sn},bn=Math.sqrt,pn=Math.pow,gn=Math.min,vn=Math.max,mn=Math.atan2,yn=Math.abs,wn=Math.cos,kn=Math.sin,Mn=Math.exp,Nn=Math.PI,m={cool:function(){return Se([N.hsl(180,1,.9),N.hsl(250,.7,.4)])},hot:function(){return Se(["#000","#f00","#ff0","#fff"]).mode("rgb")}},_n={OrRd:["#fff7ec","#fee8c8","#fdd49e","#fdbb84","#fc8d59","#ef6548","#d7301f","#b30000","#7f0000"],PuBu:["#fff7fb","#ece7f2","#d0d1e6","#a6bddb","#74a9cf","#3690c0","#0570b0","#045a8d","#023858"],BuPu:["#f7fcfd","#e0ecf4","#bfd3e6","#9ebcda","#8c96c6","#8c6bb1","#88419d","#810f7c","#4d004b"],Oranges:["#fff5eb","#fee6ce","#fdd0a2","#fdae6b","#fd8d3c","#f16913","#d94801","#a63603","#7f2704"],BuGn:["#f7fcfd","#e5f5f9","#ccece6","#99d8c9","#66c2a4","#41ae76","#238b45","#006d2c","#00441b"],YlOrBr:["#ffffe5","#fff7bc","#fee391","#fec44f","#fe9929","#ec7014","#cc4c02","#993404","#662506"],YlGn:["#ffffe5","#f7fcb9","#d9f0a3","#addd8e","#78c679","#41ab5d","#238443","#006837","#004529"],Reds:["#fff5f0","#fee0d2","#fcbba1","#fc9272","#fb6a4a","#ef3b2c","#cb181d","#a50f15","#67000d"],RdPu:["#fff7f3","#fde0dd","#fcc5c0","#fa9fb5","#f768a1","#dd3497","#ae017e","#7a0177","#49006a"],Greens:["#f7fcf5","#e5f5e0","#c7e9c0","#a1d99b","#74c476","#41ab5d","#238b45","#006d2c","#00441b"],YlGnBu:["#ffffd9","#edf8b1","#c7e9b4","#7fcdbb","#41b6c4","#1d91c0","#225ea8","#253494","#081d58"],Purples:["#fcfbfd","#efedf5","#dadaeb","#bcbddc","#9e9ac8","#807dba","#6a51a3","#54278f","#3f007d"],GnBu:["#f7fcf0","#e0f3db","#ccebc5","#a8ddb5","#7bccc4","#4eb3d3","#2b8cbe","#0868ac","#084081"],Greys:["#ffffff","#f0f0f0","#d9d9d9","#bdbdbd","#969696","#737373","#525252","#252525","#000000"],YlOrRd:["#ffffcc","#ffeda0","#fed976","#feb24c","#fd8d3c","#fc4e2a","#e31a1c","#bd0026","#800026"],PuRd:["#f7f4f9","#e7e1ef","#d4b9da","#c994c7","#df65b0","#e7298a","#ce1256","#980043","#67001f"],Blues:["#f7fbff","#deebf7","#c6dbef","#9ecae1","#6baed6","#4292c6","#2171b5","#08519c","#08306b"],PuBuGn:["#fff7fb","#ece2f0","#d0d1e6","#a6bddb","#67a9cf","#3690c0","#02818a","#016c59","#014636"],Viridis:["#440154","#482777","#3f4a8a","#31678e","#26838f","#1f9d8a","#6cce5a","#b6de2b","#fee825"],Spectral:["#9e0142","#d53e4f","#f46d43","#fdae61","#fee08b","#ffffbf","#e6f598","#abdda4","#66c2a5","#3288bd","#5e4fa2"],RdYlGn:["#a50026","#d73027","#f46d43","#fdae61","#fee08b","#ffffbf","#d9ef8b","#a6d96a","#66bd63","#1a9850","#006837"],RdBu:["#67001f","#b2182b","#d6604d","#f4a582","#fddbc7","#f7f7f7","#d1e5f0","#92c5de","#4393c3","#2166ac","#053061"],PiYG:["#8e0152","#c51b7d","#de77ae","#f1b6da","#fde0ef","#f7f7f7","#e6f5d0","#b8e186","#7fbc41","#4d9221","#276419"],PRGn:["#40004b","#762a83","#9970ab","#c2a5cf","#e7d4e8","#f7f7f7","#d9f0d3","#a6dba0","#5aae61","#1b7837","#00441b"],RdYlBu:["#a50026","#d73027","#f46d43","#fdae61","#fee090","#ffffbf","#e0f3f8","#abd9e9","#74add1","#4575b4","#313695"],BrBG:["#543005","#8c510a","#bf812d","#dfc27d","#f6e8c3","#f5f5f5","#c7eae5","#80cdc1","#35978f","#01665e","#003c30"],RdGy:["#67001f","#b2182b","#d6604d","#f4a582","#fddbc7","#ffffff","#e0e0e0","#bababa","#878787","#4d4d4d","#1a1a1a"],PuOr:["#7f3b08","#b35806","#e08214","#fdb863","#fee0b6","#f7f7f7","#d8daeb","#b2abd2","#8073ac","#542788","#2d004b"],Set2:["#66c2a5","#fc8d62","#8da0cb","#e78ac3","#a6d854","#ffd92f","#e5c494","#b3b3b3"],Accent:["#7fc97f","#beaed4","#fdc086","#ffff99","#386cb0","#f0027f","#bf5b17","#666666"],Set1:["#e41a1c","#377eb8","#4daf4a","#984ea3","#ff7f00","#ffff33","#a65628","#f781bf","#999999"],Set3:["#8dd3c7","#ffffb3","#bebada","#fb8072","#80b1d3","#fdb462","#b3de69","#fccde5","#d9d9d9","#bc80bd","#ccebc5","#ffed6f"],Dark2:["#1b9e77","#d95f02","#7570b3","#e7298a","#66a61e","#e6ab02","#a6761d","#666666"],Paired:["#a6cee3","#1f78b4","#b2df8a","#33a02c","#fb9a99","#e31a1c","#fdbf6f","#ff7f00","#cab2d6","#6a3d9a","#ffff99","#b15928"],Pastel2:["#b3e2cd","#fdcdac","#cbd5e8","#f4cae4","#e6f5c9","#fff2ae","#f1e2cc","#cccccc"],Pastel1:["#fbb4ae","#b3cde3","#ccebc5","#decbe4","#fed9a6","#ffffcc","#e5d8bd","#fddaec","#f2f2f2"]},xn=0,An=Object.keys(_n);xn<An.length;xn+=1){var En=An[xn];_n[En.toLowerCase()]=_n[En]}o=_n;return N.average=function(r,f,o){void 0===f&&(f="lrgb");var e=r.length,n=e/(o=(o=void 0===o?null:o)||Array.from(new Array(e)).map(function(){return 1})).reduce(function(r,e){return r+e});if(o.forEach(function(r,e){o[e]*=n}),r=r.map(function(r){return new p(r)}),"lrgb"===f)return Xe(r,o);for(var t,a=r.shift(),u=a.get(f),c=[],i=0,l=0,h=0;h<u.length;h++)u[h]=(u[h]||0)*o[0],c.push(isNaN(u[h])?0:o[0]),"h"!==f.charAt(h)||isNaN(u[h])||(t=u[h]/180*Te,i+=Ue(t)*o[0],l+=Ve(t)*o[0]);var d=a.alpha()*o[0];r.forEach(function(r,e){var n=r.get(f);d+=r.alpha()*o[e+1];for(var t,a=0;a<u.length;a++)isNaN(n[a])||(c[a]+=o[e+1],"h"===f.charAt(a)?(t=n[a]/180*Te,i+=Ue(t)*o[e+1],l+=Ve(t)*o[e+1]):u[a]+=n[a]*o[e+1])});for(var s=0;s<u.length;s++)if("h"===f.charAt(s)){for(var b=We(l/c[s],i/c[s])/Te*180;b<0;)b+=360;for(;360<=b;)b-=360;u[s]=b}else u[s]=u[s]/c[s];return d/=e,new p(u,f).alpha(.99999<d?1:d,!0)},N.bezier=function(r){var e=Ke(r);return e.scale=function(){return Se(e)},e},N.blend=f,N.cubehelix=function(a,f,o,u,c){void 0===a&&(a=300),void 0===f&&(f=-1.5),void 0===o&&(o=1),void 0===u&&(u=1);var i,l=0;"array"===Ze(c=void 0===c?[0,1]:c)?i=c[1]-c[0]:(i=0,c=[c,c]);function e(r){var e=en*((a+120)/360+f*r),n=nn(c[0]+i*r,u),t=(0!==l?o[0]+r*l:o)*n*(1-n)/2,r=an(e),e=tn(e);return N(rn([255*(n+t*(-.14861*r+1.78277*e)),255*(n+t*(-.29227*r-.90649*e)),255*(n+1.97294*r*t),1]))}return e.start=function(r){return null==r?a:(a=r,e)},e.rotations=function(r){return null==r?f:(f=r,e)},e.gamma=function(r){return null==r?u:(u=r,e)},e.hue=function(r){return null==r?o:("array"===Ze(o=r)?0===(l=o[1]-o[0])&&(o=o[1]):l=0,e)},e.lightness=function(r){return null==r?c:(i="array"===Ze(r)?(c=r)[1]-r[0]:(c=[r,r],0),e)},e.scale=function(){return N.scale(e)},e.hue(o),e},N.mix=N.interpolate=je,N.random=function(){for(var r="#",e=0;e<6;e++)r+="0123456789abcdef".charAt(fn(16*on()));return new p(r,"hex")},N.scale=Se,N.analyze=l.analyze,N.contrast=function(r,e){r=new p(r),e=new p(e);r=r.luminance(),e=e.luminance();return e<r?(r+.05)/(e+.05):(e+.05)/(r+.05)},N.deltaE=function(r,e,n,t,a){void 0===n&&(n=1),void 0===t&&(t=1),void 0===a&&(a=1);function f(r){return 2*Nn*r/360}var o=function(r){return 360*r/(2*Nn)};r=new p(r),e=new p(e);var u=Array.from(r.lab()),c=u[0],i=u[1],l=u[2],h=Array.from(e.lab()),d=h[0],s=h[1],b=h[2],r=(c+d)/2,u=(bn(pn(i,2)+pn(l,2))+bn(pn(s,2)+pn(b,2)))/2,e=.5*(1-bn(pn(u,7)/(pn(u,7)+pn(25,7)))),h=i*(1+e),u=s*(1+e),i=bn(pn(h,2)+pn(l,2)),s=bn(pn(u,2)+pn(b,2)),e=(i+s)/2,l=o(mn(l,h)),h=o(mn(b,u)),o=0<=l?l:l+360,b=0<=h?h:h+360,u=180<yn(o-b)?(o+b+360)/2:(o+b)/2,l=1-.17*wn(f(u-30))+.24*wn(f(2*u))+.32*wn(f(3*u+6))-.2*wn(f(4*u-63)),h=yn(h=b-o)<=180?h:b<=o?h+360:h-360;h=2*bn(i*s)*kn(f(h)/2);c=d-c,s-=i,i=1+.015*pn(r-50,2)/bn(20+pn(r-50,2)),r=1+.045*e,l=1+.015*e*l,u=30*Mn(-pn((u-275)/25,2)),u=-(2*bn(pn(e,7)/(pn(e,7)+pn(25,7))))*kn(2*f(u)),l=bn(pn(c/(n*i),2)+pn(s/(t*r),2)+pn(h/(a*l),2)+s/(t*r)*u*(h/(a*l)));return vn(0,gn(100,l))},N.distance=function(r,e,n){void 0===n&&(n="lab"),r=new p(r),e=new p(e);var t,a=r.get(n),f=e.get(n),o=0;for(t in a){var u=(a[t]||0)-(f[t]||0);o+=u*u}return Math.sqrt(o)},N.limits=l.limits,N.valid=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];try{return new(Function.prototype.bind.apply(p,[null].concat(r))),!0}catch(r){return!1}},N.scales=m,N.colors=be,N.brewer=o,N});
